from ..resource import Resource


class UserCompanyResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/users-companies')
        self.resource_name = 'UserCompany'

    def list(self, user_id=None, company_id=None, page=None):
        return self._list(user_id=user_id, company_id=company_id, page=page)

    def create(self, user_id=None, company_id=None):
        return self._create(user_id=user_id, company_id=company_id)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def delete(self, entity_id):
        return self._delete(entity_id)



from datetime import date, datetime
from pydash.chaining import chain
from pydash.objects import map_values
from pydash.strings import camel_case, snake_case
import requests

from . import __version__
from .error import HudAiError
from .resources import *

USER_AGENT = 'HUD.ai Python v{} +(https://github.com/FoundryAI/hud-ai-python#readme)'.format(__version__)

class HudAi(object):
    """
    API Client for HUD.ai that handles the API token injection and translation to/from Python
    objects
    """
    def __init__(self, api_key=None, base_url='https://api.hud.ai/v1'):
        if not api_key:
            raise HudAiError('missing api_key', 'initialization_error')

        self._api_key = api_key
        self._base_url = base_url

        self.article_highlights = ArticleHighlightsResource(self)
        self.article_key_term = ArticleKeyTermResource(self)
        self.article = ArticleResource(self)
        self.company_key_term = CompanyKeyTermResource(self)
        self.company = CompanyResource(self)
        self.domain = DomainResource(self)
        self.key_term = KeyTermResource(self)
        self.system_event = SystemEventResource(self)
        self.system_task = SystemTaskResource(self)
        self.text_corpus = TextCorpusResource(self)
        self.user_company = UserCompanyResource(self)
        self.user_digest_subscription = UserDigestSubscriptionResource(self)
        self.user_key_term = UserKeyTermResource(self)
        self.user = UserResource(self)


    def http_get(self, path, query_params={}):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.get(self._build_url(path),
                                params=self._jsonify(query_params),
                                headers=self._get_headers())

        return self._pythonify(response.json())

    def http_post(self, path, query_params={}, data={}):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.post(self._build_url(path),
                                 params=self._jsonify(query_params),
                                 data=self._jsonify(data),
                                 headers=self._get_headers())

        return self._pythonify(response.json())


    def http_put(self, path, query_params={}, data={}):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.put(self._build_url(path),
                                params=self._jsonify(query_params),
                                data=self._jsonify(data),
                                headers=self._get_headers())

        return self._pythonify(response.json())


    def http_patch(self, path, query_params={}, data={}):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.patch(self._build_url(path),
                                  params=self._jsonify(query_params),
                                  data=self._jsonify(data),
                                  headers=self._get_headers())

        return self._pythonify(response.json())


    def http_delete(self, path, query_params={}):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.delete(self._build_url(path),
                                   params=self._jsonify(query_params),
                                   headers=self._get_headers())

        return self._pythonify(response.json())


    def _build_url(self, path):
        return '{}{}'.format(self._base_url, path)


    def _get_headers(self):
        return { 'User-Agent': USER_AGENT, 'x-api-key': self._api_key }


    def _jsonify(self, value):
        if not isinstance(value, dict):
            return self._web_safe(value)

        return chain(value) \
            .map_keys(lambda value, key: camel_case(key)) \
            .map_values(lambda value: self._jsonify(value)) \
            .value()


    def _pythonify(self, value):
        if isinstance(value, str):
            try:
                return datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
            except ValueError:
                return value

        if isinstance(value, list):
            return [self._pythonify(item) for item in value]

        if isinstance(value, dict):
            return chain(value) \
                .map_keys(lambda value, key: snake_case(key)) \
                .map_values(lambda value: self._pythonify(value)) \
                .value()

        return value


    def _web_safe(self, value):
        if isinstance(value, datetime):
            return value.isoformat()

        if isinstance(value, date):
            return value.isoformat()

        if isinstance(value, list):
            return [self._web_safe(item) for item in value]

        if isinstance(value, dict):
            return map_values(value, lambda element: self._web_safe(element))

        return value



"""
Python wrapper for HUD.ai data access layer

Example usage:

from hudai.client import HudAi

client = HudAi()
"""

__package_name__ = 'hudai'
__description__ = ('HUD.ai API python bindings')
__keywords__ = ['hud.ai','hudai','foundry.ai','foundrydc']
__url__ = 'https://github.com/FoundryAI/hud-ai-python'
__version__ = '2.2.5'
__author__ = 'HUD.ai Engineering'
__email__ = 'engineering@hud.ai'
__license__ = 'MIT'



from ..resource import Resource


class UserResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/users')
        self.resource_name = 'User'


    # Core CRUD Actions


    def list(self, email=None, page=None):
        return self._list(email=email, page=page)

    def create(self, email=None, name=None, time_zone=None):
        return self._create(email=email, name=name, time_zone=time_zone)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, email=None, name=None, time_zone=None):
        return self._update(entity_id,
                            email=email,
                            name=name,
                            time_zone=time_zone)

    def delete(self, entity_id):
        return self._delete(entity_id)


    # Convenience management of their followed KeyTerms


    def followed_terms_list(self, entity_id):
        return self.http_get('/{id}/followed-terms',
                             params={'id': entity_id})

    def followed_terms_add(self, entity_id, term):
        return self.http_post('/{id}/followed-terms',
                              params={'id': entity_id},
                              data={'term': term})

    def followed_terms_remove(self, entity_id, term):
        return self.http_delete('/{id}/followed-terms/{term}',
                                params={'id': entity_id, 'term': term})


    # Convenience management of their followed Companies


    def followed_companies_list(self, entity_id):
        return self.http_get('/{id}/followed-companies',
                             params={'id': entity_id})

    def followed_companies_add(self, entity_id, company_id):
        return self.http_post('/{id}/followed-companies',
                              params={'id': entity_id},
                              data={'company_id': company_id})

    def followed_companies_remove(self, user_id, company_id):
        return self.http_delete('/{user_id}/followed-companies/{company_id}',
                                params={
                                    'user_id': user_id,
                                    'company_id': company_id
                                })



from ..resource import Resource


class UserDigestSubscriptionResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/user-digest-subscription')
        self.resource_name = 'UserDigestSubscription'

    def list(self, user_id=None, day_of_week=None, iso_hour=None, page=None):
        return self._list(user_id=user_id,
                          day_of_week=day_of_week,
                          iso_hour=iso_hour,
                          page=page)

    def create(self, user_id=None, day_of_week=None, iso_hour=None):
        return self._create(user_id=user_id,
                            day_of_week=day_of_week,
                            iso_hour=iso_hour)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, day_of_week=None, iso_hour=None):
        return self._update(entity_id, day_of_week=day_of_week, iso_hour=iso_hour)

    def delete(self, entity_id):
        return self._delete(entity_id)



from ..resource import Resource


class UserKeyTermResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/user-key-terms')
        self.resource_name = 'UserKeyTerm'

    def list(self, user_id=None, term=None, page=None):
        return self._list(user_id=user_id, term=term, page=page)

    def create(self, user_id=None, term=None):
        return self._create(user_id=user_id, term=term)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def delete(self, entity_id):
        return self._delete(entity_id)



from ..resource import Resource


class SystemTaskResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/system-tasks')
        self.resource_name = 'SystemTask'

    def list(self,
             page=None,
             started_after=None,
             started_before=None,
             completed=None):
        return self._list(
            page=page,
            started_after=started_after,
            started_before=started_before,
            completed=completed
        )

    def create(self, entity_id=None, started_at=None, completed_at=None):
        return self._create(
            id=entity_id,
            started_at=started_at,
            completed_at=completed_at
        )

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, started_at=None, completed_at=None):
        return self._update(entity_id,
            started_at=started_at,
            completed_at=completed_at
        )

    def delete(self, entity_id):
        return self._delete(entity_id)



"""HUD.ai Client Specs

Ensure that the client continues to act in a predictable way so that it can be
extended via the Resources or used directly to perform the required translations
and inject any required headers
"""

from datetime import datetime
import pytest
import requests

from hudai.client import HudAi
from hudai.resources import *


def test_initialization():
    client = HudAi(api_key='mock-api-key')

    assert isinstance(client, HudAi)
    assert isinstance(client.article_highlights, ArticleHighlightsResource)
    assert isinstance(client.article_key_term, ArticleKeyTermResource)
    assert isinstance(client.article, ArticleResource)
    assert isinstance(client.company_key_term, CompanyKeyTermResource)
    assert isinstance(client.company, CompanyResource)
    assert isinstance(client.domain, DomainResource)
    assert isinstance(client.key_term, KeyTermResource)
    assert isinstance(client.system_event, SystemEventResource)
    assert isinstance(client.system_task, SystemTaskResource)
    assert isinstance(client.text_corpus, TextCorpusResource)
    assert isinstance(client.user_company, UserCompanyResource)
    assert isinstance(client.user_key_term, UserKeyTermResource)
    assert isinstance(client.user, UserResource)


@pytest.mark.parametrize('http_verb', [('get'), ('post'), ('put'), ('patch'), ('delete')])
def test_required_parameter_injection(mocker, http_verb):
    client = HudAi(api_key='mock-api-key')
    mocker.patch.object(requests, http_verb)

    # Actual function call, e.g. client.get(path, params)
    function_under_test = getattr(client, "http_{}".format(http_verb))
    requests_function = getattr(requests, http_verb)

    assert callable(function_under_test)

    function_under_test('/test/url')

    assert requests_function.call_count == 1

    args, kwargs = requests_function.call_args

    assert args[0] == 'https://api.hud.ai/v1/test/url'
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'


@pytest.mark.parametrize('http_verb', [('get'), ('delete')])
def test_passing_requests_params(mocker, http_verb):
    client = HudAi(api_key='mock-api-key')
    mocker.patch.object(requests, http_verb)

    # Actual function call, e.g. client.get(path, params)
    function_under_test = getattr(client, "http_{}".format(http_verb))
    requests_function = getattr(requests, http_verb)

    function_under_test('/test/url', query_params={'foo_bar':'baz'})

    _, kwargs = requests_function.call_args

    assert kwargs['params'] == {'fooBar':'baz'}


@pytest.mark.parametrize('http_verb', [('post'), ('put'), ('patch')])
def test_passing_requests_params_with_data(mocker, http_verb):
    client = HudAi(api_key='mock-api-key')
    mocker.patch.object(requests, http_verb)

    # Actual function call, e.g. client.get(path, params)
    function_under_test = getattr(client, "http_{}".format(http_verb))
    requests_function = getattr(requests, http_verb)

    function_under_test('/test/url',
                        query_params={'foo_bar':'baz'},
                        data={'fizz_buzz':{'abc':'jackson_five'}})

    _, kwargs = requests_function.call_args

    assert kwargs['params'] == {'fooBar':'baz'}
    assert kwargs['data'] == {'fizzBuzz':{'abc':'jackson_five'}}


def test_jsonifiying(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.post')

    timestamp = datetime.now()
    formatted_timestamp = timestamp.isoformat()

    client.http_post('/test/url',
                     query_params={'abc':timestamp},
                     data={'xyz':timestamp})

    _, kwargs = requests.post.call_args

    assert kwargs['params']['abc'] == formatted_timestamp
    assert kwargs['data']['xyz'] == formatted_timestamp


def test_pythonification(mocker):
    client = HudAi(api_key='mock-api-key')

    mock_json = {
        'string': 'test-string',
        'boolean': True,
        'number': 123,
        'date': '2017-07-28T15:30:08.176077',
        'array': ['test', '2017-07-28T15:30:08.176077', 123],
        'object': {
            'nestedString': 'test2',
            'nestedDate': '2017-07-28T15:30:08.176077'
        }
    }

    mock_response = requests.Response()
    mocker.patch('requests.get').return_value = mock_response
    mocker.patch.object(mock_response, 'json').return_value = mock_json

    expected_timestamp = datetime(2017, 7, 28, 15, 30, 8, 176077)

    response = client.http_get('/test/url')

    assert response['string'] == 'test-string'
    assert response['boolean']
    assert response['number'] == 123
    assert response['date'] == expected_timestamp
    assert response['array'] == ['test', expected_timestamp, 123]
    assert response['object']['nested_string'] == 'test2'
    assert response['object']['nested_date'] == expected_timestamp



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import path
from setuptools import setup, find_packages


def read(filename):
    return open(path.join(path.dirname(__file__), filename)).read()


def parse_requirements(filename):
    return [line.strip()
        for line in read(filename).strip().split('\n')
        if line.strip()]


pkg = {}
exec(read('hudai/__init__.py'), pkg)

requirements = parse_requirements('requirements.txt')


setup(
    name = pkg['__package_name__'],
    version = pkg['__version__'],
    url = pkg['__url__'],
    license = pkg['__license__'],
    author = pkg['__author__'],
    author_email = pkg['__email__'],
    description = pkg['__description__'],
    packages = find_packages(exclude=['*.test', '*.test.*', 'test.*', 'test']),
    install_requires = requirements,
    download_url = '{}/archive/{}.tar.gz'.format(pkg['__url__'], pkg['__version__']),
    keywords = pkg['__keywords__'],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7'
    ],
)



"""
hudai.resources.domain
"""
from ..resource import Resource


class DomainResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/domains')
        self.resource_name = 'Domain'

    def list(self, company_id=None, hostname=None, page=None):
        return self._list(company_id=company_id, hostname=hostname, page=page)

    def create(self, company_id=None, hostname=None):
        return self._create(company_id=company_id, hostname=hostname)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, company_id=None, hostname=None):
        return self._update(entity_id, company_id=company_id, hostname=hostname)

    def delete(self, entity_id):
        return self._delete(entity_id)
